#! /bin/sh
#
# Copyright 2011 Chris Johns <chrisj@rtems.org>
#
# NIOS tools for RTEMS.
#

base=$(dirname $0)
if [ $(echo ${base} | sed -e 's/^\/.*/yes/') != yes ]; then
  base=$(pwd)/${base}
fi

#
# Log all the build output.
#
log=l.txt

#
# Versions of the tools.
#
version=11.0
binutils="binutils-2.20"
gcc="gcc-4.1"
newlib="newlib-1.18.0"

#
# Set up the default options.
#
target=nios2-rtems4.10
prefix=${base}/nios2-rtems-${version}

while [ $# -ne 0 ];
do
  case $1 in
    -p)
     shift
     prefix=$1
     ;;
  esac
  shift
done

#
# Sources
#
sources=${base}/archive
altera_nios="nios2_gnu_gcc4_11.0.tgz"
binutils_patch="binutils-2.20-rtems-20110703.diff"
gcc_patch="gcc-4.1-rtems-20110712.diff"
newlib_patch="newlib-1.18.0-rtems-20110214.diff"

#
# The detected host platform.
#
case $(uname) in
  Darwin)
   host=i386-apple-darwin10.6.0
   build=i386-apple-darwin10.6.0
   has_ln=yes
   CC="/usr/bin/gcc -O2 -fasynchronous-unwind-tables"
   cpus="4"
   make="make -j${cpus} -w"
   ;;
  MINGW32*)
   host=i686-pc-mingw32
   build=i686-pc-mingw32
   has_ln=no
   CC="gcc -O2"
   cpus="2"
   make="make -j${cpus} -w"
   ;;
  *)
   echo "error: no host support"
   exit 1
esac

make="make -w"
patch="patch"
tar="tar"

#
# Configure commands.
#
configure_binu="${binutils}/configure --build=${build} --host=${host} --target=${target} --disable-werror \
               --prefix=${prefix} --verbose --disable-nls --without-included-gettext --disable-win32-registry"
configure_gcc="${gcc}/configure --target=${target} --disable-werror \
               --prefix=${prefix} --disable-libstdcxx-pch --with-gnu-as --with-gnu-ld \
               --verbose --with-newlib --with-system-zlib --disable-nls --without-included-gettext \
               --disable-win32-registry --enable-version-specific-runtime-libs --enable-threads --disable-lto \
               --disable-plugin --enable-languages=c,c++"
newlib_cfg="normalc"

#
# The script for real.
#
export PATH=${prefix}/bin:$PATH

top=$(pwd)
build=build-${target}
log=${top}/${build}/${log}

#
# Log and echo.
#
le()
{
  eval echo "$*" >> ${log}
  eval echo "$*"
}

#
# Log, echo and run.
#
ler()
{
  eval echo "$*" >> ${log}
  eval echo "$*"
  eval $* >> ${log} 2>&1
  if [ $? -ne 0 ]; then
    eval echo "error: build failed, check log for details" >> ${log}
    tail -2 ${log}
    exit $?
  fi
}

rm -rf ${build}
mkdir ${build}

#
# Check if a build exists
#
if [ -e ${top}/${build} ]; then
  echo "warning: build output exists"
else
  mkdir -p ${top}/${build}
fi

#
# Set up the log file.
#
echo "RTEMS NIOS ${version} Build"
echo "RTEMS NIOS ${version} Build" > ${log}

le '" binutils: ${binutils}"'
le '"      gcc: ${gcc}"'
le '"   newlib: ${newlib}"'
le '"    pefix: ${prefix}"'
le '"     host: ${host}"'

hash -r

ler mkdir -p ${version}
ler cd ${version}

ler ${tar} zxf ${sources}/${altera_nios}
ler ${tar} zxf ${sources}/${newlib}.tar.gz

ler cd ${binutils}
ler ${patch} -p1 < ${sources}/${binutils_patch}
ler cd ..
ler cd ${gcc}
ler ${patch} -p1 < ${sources}/${gcc_patch}
ler cd ..
ler cd ${newlib}
ler ${patch} -p1 < ${sources}/${newlib_patch}
ler cd ..

if [ ${has_ln} == yes ]; then
  ler rm -f ${gcc}/newlib
  ler ln -s ../${newlib}/newlib ${gcc}/
else
  ler rm -rf ${gcc}/newlib
  ler cp -r ${newlib}/newlib ${gcc}/
fi

echo "RTEMS (chrisj@rtems.org) NIOS2 ${gcc}-${version}/${newlib}" > ${gcc}/gcc/DEV-PHASE
ler cd ${gcc}
ler contrib/gcc_update --touch

ler cd ${top}
ler cd ${build}

ler mkdir binutils
ler cd binutils
ler 'CC=${CC}' ../../${version}/${configure_binu}

#
# Remove the broke attempt to build pdf and html doco.
#
ler mv Makefile Makefile.orig
cat Makefile.orig | sed -e 's/^install-pdf\:.*/install-pdf: /' \
                        -e 's/^install-html\:.*/install-html: /' > Makefile
ler ${make} all
ler ${make} install

hash -r

ler cd ${top}/${build}
ler mkdir gcc
ler cd gcc
ler mkdir -p gcc/include
ler cp ${top}/${version}/${gcc}/gcc/gsyslimits.h gcc/include/syslimits.h

#
# The path to configure must always be relative or the build breaks on MinGW.
#
ler 'CC=${CC}' 'N2GNU_BUILD_NEWLIB=${newlib_cfg}' 'CFLAGS="-O2 -g"' ../../${version}/${configure_gcc}
ler ${make} N2GNU_BUILD_NEWLIB=${newlib_cfg} 'CFLAGS="-O2 -g"' all

ler cd ${top}/${build}
ler cd gcc

#
# Filter the doc stuff we do not want so it builds without error.
#
ler echo "Filtering Makefiles to remove broken doc building"
for m in $(find . -name Makefile)
do
  mv $m $m.orig
  le "Doc filteirng $m.orig -> $m"
  cat $m.orig | sed -e 's/ pdf / /g' \
                    -e 's/ pdf$/ /g' \
                    -e 's/ install-pdf / /g' \
                    -e 's/ install-pdf$/ /g' \
                    -e 's/ install-html / /g' \
                    -e 's/ install-html$/ /g' \
                    -e 's/ html / /g' \
                    -e 's/ html$/ /g' > $m
done

ler ${make} N2GNU_BUILD_NEWLIB=${newlib_cfg} 'CFLAGS="-O2 -g"' install
ler cd ${top}/${build}
